/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "callback.h"
#include "resource.h"

#include "Timesheet.h"
#include "Globals.h"

#include "UIUtil.h"

/******************************************************************
 * The day date has changed, update the active form appropriately (either detail or main)
 ******************************************************************/
void
FormUpdateDay(DateType * dayDate)
{
  ControlPtr controlPtr;
  Char dateStr[20 + 1];
  FontID prevFont;
  FormPtr formPtr;
  Word length;
  RectangleType rect;

  /* display the present week commencing date */
  dateStr[0] = NULL;
  DateToDOWDMFormat((Byte) dayDate->month, (Byte) dayDate->day,
                    (Word) dayDate->year + 1920,
                    (TSAppPrefs.prefFlags & TSPrefShortDateFlag) ? TSSysDateFormat : TSSysLongDateFormat,
                    dateStr);
  /* draw the date string in bold font */
  formPtr = FrmGetActiveForm();
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, labelID_tsDate));
  prevFont = FntSetFont(boldFont);
  length = StrLen(dateStr);
  WinDrawChars(dateStr, length, controlPtr->bounds.topLeft.x, controlPtr->bounds.topLeft.y);
  /* erase the rest of the line */
  rect.topLeft.x = controlPtr->bounds.topLeft.x + FntCharsWidth(dateStr, length);
  rect.topLeft.y = controlPtr->bounds.topLeft.y;
  rect.extent.x = 150 - rect.topLeft.x;
  rect.extent.y = 11;
  WinEraseRectangle(&rect, 0);
  FntSetFont(prevFont);
}

/******************************************************************
 * Application utility functions.
 ******************************************************************/

/******************************************************************
 * Update the form's on-screen scroll up/down buttons.
 ******************************************************************/
void
ScrollButtonUpdate(FormPtr formPtr, UInt topRow, UInt tableRows, UInt totalRows)
{
  ControlPtr controlPtr;

  /* update scroll up button */
  if (topRow > 0)
    {
      /* enable scroll up if req'd */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableUpGrey));
      if (controlPtr->attr.visible)
        {
          CtlHideControl(controlPtr);   /* hide the greyed out button */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableUp));
          CtlShowControl(controlPtr);   /* show the usable button */
        }
    }
  else
    {
      /* disable scroll up */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableUp));
      CtlHideControl(controlPtr);       /* hide the usable button */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableUpGrey));
      CtlShowControl(controlPtr);       /* show the greyed out button */
    }
  /* update scroll down button */
  if (topRow + tableRows < totalRows)
    {
      /* enable scroll down if req'd */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableDownGrey));
      if (controlPtr->attr.visible)
        {
          CtlHideControl(controlPtr);   /* hide the greyed out button */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableDown));
          CtlShowControl(controlPtr);   /* show the usable button */
        }
    }
  else
    {
      /* disable scroll down */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableDown));
      CtlHideControl(controlPtr);       /* hide the usable button */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, buttonID_tsTableDownGrey));
      CtlShowControl(controlPtr);       /* show the greyed out button */
    }
}

/******************************************************************
 *
 ******************************************************************/
void
TableScroll(Int * topRow, Int tableRows, Int totalRows, Byte dir)
{
  /* Update table top row according to direction */
  if (dir == pageUpChr)
    {
      (*topRow) = (*topRow) - (tableRows - 1);
    }
  else
    {
      (*topRow) = (*topRow) + (tableRows - 1);
    }
  /* Bounds check updated table top row */
  if ((*topRow) + tableRows >= totalRows)
    {
      (*topRow) = totalRows - tableRows;
    }
  if ((*topRow) < 0)
    {
      (*topRow) = 0;
    }
}

/*** End of file ***/