/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "callback.h"
#include "resource.h"

#include "Timesheet.h"
#include "Globals.h"
#include "Database.h"

#include "WeeklyForm.h"
#include "UIUtil.h"

/******************************************************************
 *
 ******************************************************************/
Boolean
WeeklyFormEventHandler(EventPtr event)
{
  SWord day;
  SWord month;
  SWord year;
  VoidHand hand;
  FormPtr formPtr;
  Boolean handled = false;

  if (event->eType == ctlSelectEvent)
    {
      /* A control button was clicked */
      if (event->data.ctlEnter.controlID == buttonID_tsDayViewButton)
        {
          /* Change to the main daily view */
          FrmGotoForm(formID_tsMain);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsNextWeek)
        {
          /* Select the next week */
          WeeklyFormChangeDate(7);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsPrevWeek)
        {
          /* Select the previous week */
          WeeklyFormChangeDate(-7);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsGoto)
        {
          /* Jump to user specified date */
          day = TSPresentDayDate.day;
          month = TSPresentDayDate.month;
          year = TSPresentDayDate.year + 1920;
          hand = DmGet1Resource('tSTR', stringID_tsSelectDate);
          if (SelectDay(selectDayByWeek, &month, &day, &year, (CharPtr) MemHandleLock(hand)))
            {
              /* User has entered date to go to, save the new date */
              TSPresentDayDate.day = day;
              TSPresentDayDate.month = month;
              TSPresentDayDate.year = year - 1920;
              /* Find the day record for today (if any) */
              FindDayRec(&TSPresentDayDate, &TSPresentDayRecIdx, &TSPresentDayRecExists);
              /* Reinitialise the weekly form (this is a cheap way of getting display updated) */
              WeeklyFormInit();
              /* Update the weekly form GUI */
              WeeklyFormUpdateWeek();
              FormUpdateDay(&TSPresentSummaryDate);
              WeeklyFormTableDraw();
            }
          MemHandleUnlock(hand);
          DmReleaseResource(hand);
          handled = true;
        }
    }
  else if (event->eType == ctlRepeatEvent)
    {
      if (event->data.ctlSelect.controlID == buttonID_tsTableUp)
        {
          /* Scroll table up */
          WeeklyFormTableScroll(pageUpChr);
          WeeklyFormTableDraw();
        }
      else if (event->data.ctlSelect.controlID == buttonID_tsTableDown)
        {
          /* Scroll table down */
          WeeklyFormTableScroll(pageDownChr);
          WeeklyFormTableDraw();
        }
    }
  else if (event->eType == menuEvent)
    {
      /* A menu item has been selected */
      if (event->data.menu.itemID == menuitemID_tsGotoEarliest)
        {
          /* User wants to go to earliest day in database */
          WeeklyFormGotoEarliestWeek();
        }
      else if (event->data.menu.itemID == menuitemID_tsGotoLatest)
        {
          /* User wants to go to latest day in database */
          WeeklyFormGotoLatestWeek();
        }
      else if (event->data.menu.itemID == menuitemID_tsEraseWeek)
        {
          hand = DmGet1Resource('tSTR', stringID_tsWeek);
          if (GetNumSummaryRecords() > 1)
            {
              /* User wants to erase the current summary week */
              if (FrmCustomAlert(alertID_tsConfirmDayWeekDel, 
                  (CharPtr)MemHandleLock(hand), NULL, NULL) == 0)
              {
                /* User confirms delete, delete all entries for this week */
                WeeklyFormEraseWeek();
              }
            }
          else
            {
              /* Nothing in this week to erase */
              FrmCustomAlert(alertID_tsEmptyDayWeek, (CharPtr)MemHandleLock(hand), NULL, NULL);
            }
          MemHandleUnlock(hand);
          DmReleaseResource(hand);        
        }
      else if (event->data.menu.itemID == menuitemID_tsPref)
        {
          /* display the prefs form */
          FrmPopupForm(formID_tsPref);
          handled = true;
        }
      else if (event->data.menu.itemID == menuitemID_tsAbout)
        {
          /* display the About form */
          FrmPopupForm(formID_tsAbout);
          /* FrmHelp(helpID_tsAbout); */
          handled = true;
        }
    }
  else if (event->eType == keyDownEvent)
    {
      /* Hardware key has been pressed, is it one app should respond to? */
      if (event->data.keyDown.chr == pageUpChr)
        {
          /* Command key up, goto previous week */
          WeeklyFormChangeDate(-7);
          handled = true;
        }
      else if (event->data.keyDown.chr == pageDownChr)
        {
          /* Command key down, goto next week */
          WeeklyFormChangeDate(7);
          handled = true;
        }
    }
  else if (event->eType == tblEnterEvent)
    {
      /* Do nothing, stops UI from inverting tapped columns */
      handled = true;
    }
  else if (event->eType == frmOpenEvent)
    {
      /* The Weekly form has been opened */
      formPtr = FrmGetActiveForm();
      FrmDrawForm(formPtr);
      /* Initialise the weekly form */
      WeeklyFormInit();
      /* Draw the weekly form */
      WeeklyFormUpdateWeek();
      FormUpdateDay(&TSPresentSummaryDate);
      WeeklyFormTableDraw();
      handled = true;
    }
  return handled;
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormUpdateWeek(void)
{
  FormPtr formPtr;
  ControlPtr controlPtr;
  Word weekNum;
  Char weekTxt[4 + 1];
  FontID prevFont;
  RectangleType controlRect;

  formPtr = FrmGetActiveForm();
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, labelID_tsWeek));
  /* Get the week number of the current week (thanks to Palm Computing) */
  weekNum = GetWeekNumber(TSPresentSummaryDate.month, TSPresentSummaryDate.day, TSPresentSummaryDate.year + 1920);
  /* Erase the old week number */
  MemMove(&controlRect, &(controlPtr->bounds), sizeof(RectangleType));
  /* Manually set extents based on font size */
  controlRect.extent.x = 10;
  controlRect.extent.y = 11;
  WinEraseRectangle(&controlRect, 0);
  /* Display the week number */
  StrIToA(weekTxt, weekNum);
  prevFont = FntSetFont(stdFont);
  WinDrawChars(weekTxt, StrLen(weekTxt), controlPtr->bounds.topLeft.x, controlPtr->bounds.topLeft.y);
  FntSetFont(prevFont);
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormInit(void)
{
  DateType startDate;
  DateType endDate;
  Int weekDay;

  /* Find the date for the start of this week */
  weekDay = DayOfWeek(TSPresentDayDate.month, TSPresentDayDate.day, TSPresentDayDate.year + 1920);
  MemMove(&startDate, &TSPresentDayDate, sizeof(DateType));
  /* Adjust to week start depending on palm system settings */
  if (TSSysWeekStartDay == 0)
  {
        /* Week starts on Sunday */
    DateAdjust(&startDate, -weekDay);   
  } 
  else
  {
        /* Week starts on Monday */
    DateAdjust(&startDate, -( weekDay == 0 ? 6 : weekDay - 1));
  }
  /* Set global summary date for week start */
  MemMove(&TSPresentSummaryDate, &startDate, sizeof(DateType));
  /* Find the date for the end of this week */
  MemMove(&endDate, &startDate, sizeof(DateType));
  DateAdjust(&endDate, 6);
  /* Build a summary database */
  BuildSummaryDatabase(&startDate, &endDate);
  /* Select the week view button */
  FrmSetControlGroupSelection(FrmGetActiveForm(), groupID_tsViewGroup, buttonID_tsWeekViewButton);
  /* Initialise the table globals */
  TSTableTopEntry = 0;
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormGotoEarliestWeek(void)
{
  /* Find the earliest day rec (if one exists) */
  if (FindEarliestDayRec())
    {
      /* Find the day record for today (if any) */
      FindDayRec(&TSPresentDayDate, &TSPresentDayRecIdx, &TSPresentDayRecExists);
      /* Reinitialise the weekly form (this is a cheap way of getting display updated) */
      WeeklyFormInit();
      /* Update the weekly form GUI */
      WeeklyFormUpdateWeek();
      FormUpdateDay(&TSPresentSummaryDate);
      WeeklyFormTableDraw();
    }
  else
    {
      /* no earlier day to go to, alert user */
      FrmAlert(alertID_tsEmptyDB);
    }
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormGotoLatestWeek(void)
{
  /* Find the earliest day rec (if one exists) */
  if (FindLatestDayRec())
    {
      /* Find the day record for today (if any) */
      FindDayRec(&TSPresentDayDate, &TSPresentDayRecIdx, &TSPresentDayRecExists);
      /* Reinitialise the weekly form (this is a cheap way of getting display updated) */
      WeeklyFormInit();
      /* Update the weekly form GUI */
      WeeklyFormUpdateWeek();
      FormUpdateDay(&TSPresentSummaryDate);
      WeeklyFormTableDraw();
    }
  else
    {
      /* no earlier day to go to, alert user */
      FrmAlert(alertID_tsEmptyDB);
    }
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormChangeDate(Int deltaDays)
{
  /* Modify the global dates accordingly */
  DateAdjust(&TSPresentDayDate, deltaDays);
  /* Find the day record for today (if any) */
  FindDayRec(&TSPresentDayDate, &TSPresentDayRecIdx, &TSPresentDayRecExists);
  /* Reinitialise the weekly form (this is a cheap way of getting display updated) */
  WeeklyFormInit();
  /* Update the weekly form GUI */
  WeeklyFormUpdateWeek(); 
  FormUpdateDay(&TSPresentSummaryDate);
  WeeklyFormTableDraw();
}

/******************************************************************
 * Erase all entries in the current week.
 ******************************************************************/
void WeeklyFormEraseWeek(void)
{
  DateType date;
  LocalID summaryID;
  Int summaryCardNum;  
  DmOpenRef summaryDB;  

  /* Find the week end date */
  MemMove(&date, &TSPresentSummaryDate, sizeof(DateType));
  DateAdjust(&date, 6);
  EraseDatabaseRange(&TSPresentSummaryDate, &date);
  /* Check if there's an existing summary database */
  summaryDB = DmOpenDatabaseByTypeCreator(TSSummaryDBType, TSAppID, dmModeReadWrite);
  if (summaryDB)
    {
      /* Get locaID, cardID etc. so we can delete database */
      DmOpenDatabaseInfo(summaryDB, &summaryID, NULL, NULL, &summaryCardNum, NULL);
      /* Close and delete existing summary database */
      DmCloseDatabase(summaryDB);
      DmDeleteDatabase(summaryCardNum, summaryID);
    }
  /* Nothing to display, move table to top row */
  TSTableTopEntry = 0;
  /* Redraw the main table */
  WeeklyFormTableDraw();
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormTableDrawCell(VoidPtr tablePtr, Word row, Word column, RectanglePtr bounds)
{
  Char txt[25 + 1];
  UInt idx;
  SWord txtWidth;  
  SWord txtLen;
  Boolean txtFitsInWidth;
  FontID prevFont;
  VoidHand listHand;
  TSCatRecType *listPtr;
  RectangleType rowRect =
  {
    {0, 0},             /* WARNING: note these are hardcoded bounds! */
    {159, 11}
  };                    

  /* Start fixup from the GNU Palmpilot SDK FAQ so callback function can access global variables */
  CALLBACK_PROLOGUE;

  /* Get the entry string index from the item */
  idx = TblGetItemInt(tablePtr, row, column);
  /* Format the column appropriately (either text or HOURmin) */
  if (column == 0)
    {
      /* Erase the entire row once before we draw the first column. This reduces screen flicker, and is more efficient
       * than erasing each individual cell before re-filling it. */
      rowRect.topLeft.x = bounds->topLeft.x;
      rowRect.topLeft.y = bounds->topLeft.y;
      WinEraseRectangle(&rowRect, 0);
      /* Draw the hour in the large font */
      prevFont = FntSetFont(boldFont);
      /* Check we have some hours to show */
      if (idx & 0x0FFF)
        {
          StrIToA(txt, idx & 0x0FFF);
          txtLen = StrLen(txt);
          WinDrawChars(txt, txtLen, bounds->topLeft.x + (18 - (6 * txtLen)), bounds->topLeft.y);
        }
      /* Draw the minutes in the normal font */
      txt[0] = 48 + ((idx & 0xE000) >> 13);
      txt[1] = 48 + (5 * ((idx & 0x1000) >> 12));
      txt[2] = NULL;
      FntSetFont(stdFont);
      WinDrawChars(txt, 2, bounds->topLeft.x + 18, bounds->topLeft.y);
    }
  else
    {
      /* Get the appropriate category record */
      listHand = DmQueryRecord(TSDatabase, column);
      listPtr = MemHandleLock(listHand);

      /* Get the string to put in the cell */
      StrCopy(txt, listPtr->cats[idx]);
      MemHandleUnlock(listHand);

      /* Trim the client string to fit within the cell */
      prevFont = FntSetFont(stdFont);

      /* Draw the cell contents */
      txtWidth = bounds->extent.x;
      txtLen = StrLen( txt );
      FntCharsInWidth( txt, &txtWidth, &txtLen, &txtFitsInWidth );
      txt[ txtLen ] = NULL;
      WinDrawChars(txt, txtLen, bounds->topLeft.x, bounds->topLeft.y);
    }
  /* Restore the previous font */
  FntSetFont(prevFont);

  /* End fixup from the GNU Palmpilot SDK FAQ so callback function can access global variables */
  CALLBACK_EPILOGUE;
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormTableDraw(void)
{
  FormPtr formPtr;
  TablePtr tablePtr;
  ControlPtr controlPtr;
  DmOpenRef summaryDB;
  Int numSummaryRecs;
  Int recIdx;
  UInt row;
  UInt col;
  VoidHand recHand;
  TSSummaryRecType *recPtr;
  TSSummaryTotalRecType *summaryTotalPtr;
  UInt totalHours;
  Byte totalMins;
  UInt totalChargeableHours;
  Byte totalChargeableMins;  
  Char txt[3];
  Byte txtLen;

  /* Get a pointer to the weekly form as it may not be the visible one at the moment */
  formPtr = FrmGetFormPtr(formID_tsWeekly);
  /* Get the table pointer */
  tablePtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, tableID_tsMain));
  /* Check if there's an existing summary database to fill table with */
  numSummaryRecs = GetNumSummaryRecords();
  /* Open summary database, this may return a NULL reference if no summary database exists */
  summaryDB = DmOpenDatabaseByTypeCreator(TSSummaryDBType, TSAppID, dmModeReadWrite);
  /* Skip to first record that currently appears at the top of the table */
  recIdx = TSTableTopEntry;
  for (row = 0; row < TSSummaryFormTableRows; row++, recIdx++)
    {
      if (recIdx < numSummaryRecs - 1 /* last record contains total hours/mins */ )
        {
          /* Have a record to display */
          recHand = DmQueryRecord(summaryDB, recIdx);
          recPtr = MemHandleLock(recHand);

          /* Set summary hours and minutes */
          TblSetItemStyle(tablePtr, row, 0, customTableItem);
          TblSetItemInt(tablePtr, row, 0, recPtr->totalDuration);

          /* Set client column */
          TblSetItemStyle(tablePtr, row, 1, customTableItem);
          /* Translate the client entry index into the actual client index using the client trans table */
          TblSetItemInt(tablePtr, row, 1, recPtr->catIdx[0]);

          /* Set project column */
          TblSetItemStyle(tablePtr, row, 2, customTableItem);
          /* Translate the client entry index into the actual client index using the client trans table */
          TblSetItemInt(tablePtr, row, 2, recPtr->catIdx[1]);

          /* Set task column */
          TblSetItemStyle(tablePtr, row, 3, customTableItem);
          /* Translate the client entry index into the actual client index using the client trans table */
          TblSetItemInt(tablePtr, row, 3, recPtr->catIdx[2]);

          /* Finished setting up columns */
          MemHandleUnlock(recHand);
          /* Data for this row, make it usable and selectable */
          TblSetRowUsable(tablePtr, row, true);
          /* Mark the row invalid, so it re-draws */
          TblMarkRowInvalid(tablePtr, row);
        }
      else
        {
          /* No data for this row, make it unusable */
          TblSetRowUsable(tablePtr, row, false);
        }
    }
  /* Display last record totals */
  if (summaryDB)
    {
      recHand = DmQueryRecord(summaryDB, numSummaryRecs - 1);
      summaryTotalPtr = MemHandleLock(recHand);
      totalHours = summaryTotalPtr->totalHours;
      totalMins = summaryTotalPtr->totalMins;
      totalChargeableHours = summaryTotalPtr->totalChargeableHours;
      totalChargeableMins = summaryTotalPtr->totalChargeableMins;  
      MemHandleUnlock(recHand);
      /* Finished with summary database */
      DmCloseDatabase(summaryDB);
    } 
  else    
    {
      totalHours = 0;
      totalMins = 0;
      totalChargeableHours = 0;
      totalChargeableMins = 0;
    }
  /* Make all columns in the table usable */
  for (col = 0; col < TSMainFormTableCols; col++)
    {
      TblSetColumnUsable(tablePtr, col, true);
    }
  /* Set column spacing */
  TblSetColumnSpacing(tablePtr, 0, 3);
  TblSetColumnSpacing(tablePtr, 1, 3);
  TblSetColumnSpacing(tablePtr, 2, 3);
  TblSetColumnSpacing(tablePtr, 3, 0);
  /* Set the custom drawn columns */
  TblSetCustomDrawProcedure(tablePtr, 0, WeeklyFormTableDrawCell);      /* Hour / mins column */
  TblSetCustomDrawProcedure(tablePtr, 1, WeeklyFormTableDrawCell);      /* Client column */
  TblSetCustomDrawProcedure(tablePtr, 2, WeeklyFormTableDrawCell);      /* Project column */
  TblSetCustomDrawProcedure(tablePtr, 3, WeeklyFormTableDrawCell);      /* Task column */
  /* Finally redraw the table */
  TblDrawTable(tablePtr);
  if (numSummaryRecs - 1 > 0)
    {
      /* Draw a little line to indicate it's a total */
      controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, labelID_tsTotal));
      col = controlPtr->bounds.topLeft.x;
      row = controlPtr->bounds.topLeft.y;
      WinDrawLine(col, row - 2, col + 28, row - 2);
      if (totalHours > 0)
        {
          /* Draw the total hours */
          StrIToA(txt, totalHours);
          txtLen = StrLen(txt);
          FntSetFont(boldFont);
          WinDrawChars(txt, txtLen, col + (18 - (6 * txtLen)), row);
        }
      /* Always draw the minutes in the normal font */
      FntSetFont(stdFont);
      txt[0] = 48 + ((totalMins & 0x0E) >> 1);
      txt[1] = 48 + (5 * (totalMins & 0x01));
      txt[2] = NULL;
      WinDrawChars(txt, 2, col + 18, row);
          
      /* Draw the chargeable hour and minute totals if req'd. 
       * Note, this preference is 'inverted', 1 = Off, 0 = On so it's on by default */
      if (!(TSAppPrefs.prefFlags & TSPrefChargeTotalsFlag))
        {
          /* Draw a little line to indicate it's a total */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, labelID_tsTotal));
          col = controlPtr->bounds.topLeft.x + 38;
          row = controlPtr->bounds.topLeft.y;
          WinDrawLine(col, row - 2, col + 28, row - 2);
          if (totalChargeableHours > 0)
            {
              /* Draw the total hours */
              StrIToA(txt, totalChargeableHours);
              txtLen = StrLen(txt);
              FntSetFont(boldFont);
              WinDrawChars(txt, txtLen, col + (18 - (6 * txtLen)), row);
            }
          /* Always draw the minutes in the normal font */
          FntSetFont(stdFont);
          txt[0] = 48 + ((totalChargeableMins & 0x0E) >> 1);
          txt[1] = 48 + (5 * (totalChargeableMins & 0x01));
          txt[2] = NULL;
          WinDrawChars(txt, 2, col + 18, row);
        }
    }
  /* Update the on-screen scroll buttons */
  WeeklyFormScrollButtonUpdate(formPtr);
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormTableScroll(Byte dir)
{
  Int numSummaryRecs;

  /* Check if there's an existing summary database */
  numSummaryRecs = GetNumSummaryRecords();
  if (numSummaryRecs > 0)  
    {
      numSummaryRecs --; /* Ignore last totals record */
    }
  /* Update the scroll buttons */
  TableScroll(&TSTableTopEntry, TSSummaryFormTableRows, numSummaryRecs, dir);
}

/******************************************************************
 *
 ******************************************************************/
void
WeeklyFormScrollButtonUpdate(FormPtr formPtr)
{
  Int numSummaryRecs;

  /* Check if there's an existing summary database */
  numSummaryRecs = GetNumSummaryRecords();
  if (numSummaryRecs > 0)  
    {
      numSummaryRecs --; /* Ignore last totals record */
    }
  /* Update the scroll buttons */
  ScrollButtonUpdate(formPtr, TSTableTopEntry, TSSummaryFormTableRows, numSummaryRecs);
}

/***********************************************************************
 *
 * FUNCTION:    FirstDayOfYear
 *
 * DESCRIPTION: Return the number of day from 1/1/1904 of the first day 
 *              of the year passed.
 *
 *                                       The first day of the year is always a Monday.  The rule 
 *                                       for determining the first day of the year is:
 *                              
 *                                       New Years Day  First Day of the Year
 *                                       ------------   ---------------------
 *                                       Monday                 Monday Jan 1
 *                                       Tuesday                        Monday Dec 31
 *                                       Wednesday              Monday Dec 30
 *                                       Thursday               Monday Dec 29
 *                                       Friday                 Monday Jan 4 
 *                                       Saturday               Monday Jan 3 
 *                                       Sunday                 Monday Jan 2
 *
 * PARAMETERS:   year  - year (1904-2031)
 *
 * RETURNED:     number of days since 1/1/1904
 *
 * REVISION HISTORY:
 *                      Name    Date            Description
 *                      ----    ----            -----------
 *                      art     4/4/96  Initial Revision
 *
 ***********************************************************************/
ULong
FirstDayOfYear(Word year)
{
  ULong days;
  Word dayOfWeek;
  DateType date;

  /* Get days to January 1st of the year passed. */
  date.day = 1;
  date.month = 1;
  date.year = year - firstYear;
  days = DateToDays(date);

  dayOfWeek = DayOfWeek(1, 1, year);

  /* Move to monday. */
  days++;
  days -= dayOfWeek;


  if (dayOfWeek >= friday)
    days += daysInWeek;

  return (days);
}


/***********************************************************************
 *
 * FUNCTION:    GetWeekNumber
 *
 * DESCRIPTION: Calculate the week number of the specified date.
 *
 * PARAMETERS:   month - month (1-12)
 *              day   - day (1-31)
 *              year  - year (1904-2031)
 *
 * RETURNED:     week number (1-53)
 *
 * REVISION HISTORY:
 *                      Name    Date            Description
 *                      ----    ----            -----------
 *                      art     4/4/96  Initial Revision
 *
 ***********************************************************************/
Word
GetWeekNumber(Word month, Word day, Word year)
{
  Word dow;
  Word week;
  ULong days;
  ULong firstOfYear;
  ULong firstOfNextYear;
  DateType date;

  /* Calculate the julian date of Monday in the same week as the specified date. */
  date.day = day;
  date.month = month;
  date.year = year - firstYear;
  days = DateToDays(date);

  /* Adjust the day of the week by the preference setting for the first day of the week. */
  dow = (DayOfWeek(month, day, year) - TSSysWeekStartDay + daysInWeek)
    % daysInWeek;

  if (monday < TSSysWeekStartDay)
    days -= (Short) dow - (monday + daysInWeek - TSSysWeekStartDay);
  else
    days -= (Short) dow - (monday - (Short) TSSysWeekStartDay);


  firstOfYear = FirstDayOfYear(year);

  if (days < firstOfYear)
    {
      /* The date passed is in a week that is part of the prior year, so get the start of the prior year. */
      if (year > firstYear)
        firstOfYear = FirstDayOfYear(--year);
    }
  else
    {
      /* Make sure the date passed is not in a week that in part of next year. */
      if (year < lastYear)
        {
          firstOfNextYear = FirstDayOfYear(year + 1);
          if (days == firstOfNextYear)
            firstOfYear = firstOfNextYear;
        }
    }

  week = ((Short) (days - firstOfYear)) / daysInWeek + 1;       /* one base */

  return (week);
}

/*** End of file ***/